@@ -3,7 +3,6 @@ import logging
 import os
 import socket
 import time
-from hashlib import md5
 
 from fsspec import AbstractFileSystem
 from fsspec.spec import AbstractBufferedFile
@@ -23,9 +22,6 @@ logger.addHandler(handle)
 if "S3FS_LOGGING_LEVEL" in os.environ:
     logger.setLevel(os.environ["S3FS_LOGGING_LEVEL"])
 
-logging.getLogger('boto3').setLevel(logging.WARNING)
-logging.getLogger('botocore').setLevel(logging.WARNING)
-
 try:
     from boto3.s3.transfer import S3_RETRYABLE_ERRORS
 except ImportError:
@@ -36,17 +32,6 @@ except ImportError:
 _VALID_FILE_MODES = {'r', 'w', 'a', 'rb', 'wb', 'ab'}
 
 
-def tokenize(*args, **kwargs):
-    """ Deterministic token
-
-    >>> tokenize('Hello') == tokenize('Hello')
-    True
-    """
-    if kwargs:
-        args += (kwargs,)
-    return md5(str(tuple(args)).encode()).hexdigest()
-
-
 def split_path(path):
     """
     Normalise S3 path string into bucket and key.
@@ -143,6 +128,7 @@ class S3FileSystem(AbstractFileSystem):
     read_timeout = 15
     default_block_size = 5 * 2**20
     protocol = 's3'
+    _extra_tokenize_attributes = ('default_block_size',)
 
     def __init__(self, anon=False, key=None, secret=None, token=None,
                  use_ssl=True, client_kwargs=None, requester_pays=False,
@@ -159,7 +145,6 @@ class S3FileSystem(AbstractFileSystem):
         if password:
             secret = password
 
-        super().__init__()
         self.anon = anon
         self.session = None
         self.passed_in_session = session
@@ -185,6 +170,7 @@ class S3FileSystem(AbstractFileSystem):
         self.use_ssl = use_ssl
         self.s3 = self.connect()
         self._kwargs_helper = ParamKwargsHelper(self.s3)
+        super().__init__()
 
     def _filter_kwargs(self, s3_method, kwargs):
         return self._kwargs_helper.filter_dict(s3_method.__name__, kwargs)
@@ -270,7 +256,7 @@ class S3FileSystem(AbstractFileSystem):
                 'token': cred['SessionToken'], 'anon': False}
 
     def _open(self, path, mode='rb', block_size=None, acl='', version_id=None,
-              fill_cache=None, cache_type=None, autocommit=True, **kwargs):
+              fill_cache=None, cache_type=None, autocommit=True, requester_pays=None, **kwargs):
         """ Open a file for reading or writing
 
         Parameters
@@ -299,6 +285,9 @@ class S3FileSystem(AbstractFileSystem):
         cache_type : str
             See fsspec's documentation for available cache_type values. Set to "none"
             if no caching is desired. If None, defaults to ``self.default_cache_type``.
+        requester_pays : bool (optional)
+            If RequesterPays buckets are supported.  If None, defaults to the
+            value used when creating the S3FileSystem (which defaults to False.)
         kwargs: dict-like
             Additional parameters used for s3 methods.  Typically used for
             ServerSideEncryption.
@@ -307,6 +296,8 @@ class S3FileSystem(AbstractFileSystem):
             block_size = self.default_block_size
         if fill_cache is None:
             fill_cache = self.default_fill_cache
+        if requester_pays is None:
+            requester_pays = bool(self.req_kw)
 
         acl = acl or self.s3_additional_kwargs.get('ACL', '')
         kw = self.s3_additional_kwargs.copy()
@@ -321,7 +312,7 @@ class S3FileSystem(AbstractFileSystem):
         return S3File(self, path, mode, block_size=block_size, acl=acl,
                       version_id=version_id, fill_cache=fill_cache,
                       s3_additional_kwargs=kw, cache_type=cache_type,
-                      autocommit=autocommit)
+                      autocommit=autocommit, requester_pays=requester_pays)
 
     def _lsdir(self, path, refresh=False, max_items=None):
         if path.startswith('s3://'):
@@ -412,23 +403,6 @@ class S3FileSystem(AbstractFileSystem):
             self.dircache[''] = files
         return self.dircache['']
 
-    def __getstate__(self):
-        if self.passed_in_session:
-            raise NotImplementedError
-        d = self.__dict__.copy()
-        del d['s3']
-        del d['session']
-        del d['_kwargs_helper']
-        del d['dircache']
-        logger.debug("Serialize with state: %s", d)
-        return d
-
-    def __setstate__(self, state):
-        self.__dict__.update(state)
-        self.s3 = self.connect()
-        self.dircache = {}
-        self._kwargs_helper = ParamKwargsHelper(self.s3)
-
     def _ls(self, path, refresh=False):
         """ List files in given bucket, or list of buckets.
 
@@ -511,6 +485,31 @@ class S3FileSystem(AbstractFileSystem):
                 raise ValueError('Failed to head path %r: %s' % (path, e))
         return super().info(path)
 
+    def isdir(self, path):
+        path = self._strip_protocol(path).strip("/")
+        # Send buckets to super
+        if "/" not in path:
+            return super(S3FileSystem, self).isdir(path)
+
+        if path in self.dircache:
+            for fp in self.dircache[path]:
+                # For files the dircache can contain itself.
+                # If it contains anything other than itself it is a directory.
+                if fp["name"] != path:
+                    return True
+            return False
+
+        parent = self._parent(path)
+        if parent in self.dircache:
+            for f in self.dircache[parent]:
+                if f["name"] == path:
+                    # If we find ourselves return whether we are a directory
+                    return f["type"] == "directory"
+            return False
+
+        # This only returns things within the path and NOT the path object itself
+        return bool(self._lsdir(path))
+
     def ls(self, path, detail=False, refresh=False, **kwargs):
         """ List single "directory" with or without details
 
@@ -647,7 +646,7 @@ class S3FileSystem(AbstractFileSystem):
         `Metadata Reference`_.
 
         Parameters
-        ---------
+        ----------
         kw_args : key-value pairs like field="value", where the values must be
             strings. Does not alter existing fields, unless
             the field appears here - if the value is None, delete the
@@ -919,6 +918,8 @@ class S3File(AbstractBufferedFile):
         Optional version to read the file at.  If not specified this will
         default to the current version of the object.  This is only used for
         reading.
+    requester_pays : bool (False)
+        If RequesterPays buckets are supported.
 
     Examples
     --------
@@ -937,7 +938,7 @@ class S3File(AbstractBufferedFile):
 
     def __init__(self, s3, path, mode='rb', block_size=5 * 2 ** 20, acl="",
                  version_id=None, fill_cache=True, s3_additional_kwargs=None,
-                 autocommit=True, cache_type='bytes'):
+                 autocommit=True, cache_type='bytes', requester_pays=False):
         bucket, key = split_path(path)
         if not key:
             raise ValueError('Attempt to open non key-like path: %s' % path)
@@ -951,6 +952,7 @@ class S3File(AbstractBufferedFile):
         self.parts = None
         self.fill_cache = fill_cache
         self.s3_additional_kwargs = s3_additional_kwargs or {}
+        self.req_kw = {'RequestPayer': 'requester'} if requester_pays else {}
         super().__init__(s3, path, mode, block_size, autocommit=autocommit,
                          cache_type=cache_type)
         self.s3 = self.fs  # compatibility
@@ -972,7 +974,9 @@ class S3File(AbstractBufferedFile):
                     self.details = self.fs.info(self.path)
                     self.version_id = self.details.get('VersionId')
 
+        # when not using autocommit we want to have transactional state to manage
         self.append_block = False
+
         if 'a' in mode and s3.exists(path):
             loc = s3.info(path)['size']
             if loc < 5 * 2 ** 20:
@@ -987,7 +991,7 @@ class S3File(AbstractBufferedFile):
                                 **kwargs)
 
     def _initiate_upload(self):
-        if not self.append_block and self.tell() < self.blocksize:
+        if not self.autocommit and not self.append_block and self.tell() < self.blocksize:
             # only happens when closing small file, use on-shot PUT
             return
         logger.debug("Initiate upload for %s" % self)
@@ -1053,14 +1057,14 @@ class S3File(AbstractBufferedFile):
         return self.fs.url(self.path, **kwargs)
 
     def _fetch_range(self, start, end):
-        return _fetch_range(self.fs.s3, self.bucket, self.key, self.version_id, start, end)
+        return _fetch_range(self.fs.s3, self.bucket, self.key, self.version_id, start, end, req_kw=self.req_kw)
 
     def _upload_chunk(self, final=False):
         bucket, key = split_path(self.path)
         logger.debug("Upload for %s, final=%s, loc=%s, buffer loc=%s" % (
             self, final, self.loc, self.buffer.tell()
         ))
-        if not self.append_block and final and self.tell() < self.blocksize:
+        if not self.autocommit and not self.append_block and final and self.tell() < self.blocksize:
             # only happens when closing small file, use on-shot PUT
             data1 = False
         else:
