@@ -9,7 +9,7 @@ import pytest
 from itertools import chain
 import fsspec.core
 from s3fs.core import S3FileSystem
-from s3fs.utils import seek_delimiter, ignoring, SSEParams
+from s3fs.utils import ignoring, SSEParams
 import moto
 import boto3
 from unittest import mock
@@ -94,6 +94,7 @@ def s3():
         for flist in [files, csv_files, text_files, glob_files]:
             for f, data in flist.items():
                 client.put_object(Bucket=test_bucket_name, Key=f, Body=data)
+        S3FileSystem.clear_instance_cache()
         s3 = S3FileSystem(anon=False)
         s3.invalidate_cache()
         yield s3
@@ -164,13 +165,6 @@ def test_config_kwargs():
     assert s3.connect(refresh=True).meta.config.signature_version == 's3v4'
 
 
-def test_tokenize():
-    from s3fs.core import tokenize
-    a = (1, 2, 3)
-    assert isinstance(tokenize(a), (str, bytes))
-    assert tokenize(a) != tokenize(a, other=1)
-
-
 def test_idempotent_connect(s3):
     con1 = s3.connect()
     con2 = s3.connect(refresh=False)
@@ -677,23 +671,6 @@ def test_read_small(s3):
         assert len(f.cache) < len(out)
 
 
-def test_seek_delimiter(s3):
-    fn = 'test/accounts.1.json'
-    data = files[fn]
-    with s3.open('/'.join([test_bucket_name, fn])) as f:
-        seek_delimiter(f, b'}', 0)
-        assert f.tell() == 0
-        f.seek(1)
-        seek_delimiter(f, b'}', 5)
-        assert f.tell() == data.index(b'}') + 1
-        seek_delimiter(f, b'\n', 5)
-        assert f.tell() == data.index(b'\n') + 1
-        f.seek(1, 1)
-        ind = data.index(b'\n') + data[data.index(b'\n') + 1:].index(b'\n') + 1
-        seek_delimiter(f, b'\n', 5)
-        assert f.tell() == ind + 1
-
-
 def test_read_s3_block(s3):
     data = files['test/accounts.1.json']
     lines = io.BytesIO(data).readlines()
@@ -1244,18 +1221,17 @@ def test_without_passed_in_session_set_unique(s3):
 
 
 def test_pickle_without_passed_in_session(s3):
+    import pickle
     s3 = S3FileSystem()
-    try:
-        s3.__getstate__()
-    except NotImplementedError:
-        pytest.fail("Unexpected NotImplementedError")
+    pickle.dumps(s3)
 
 
 def test_pickle_with_passed_in_session(s3):
+    import pickle
     session = boto3.session.Session()
     s3 = S3FileSystem(session=session)
-    with pytest.raises(NotImplementedError):
-        s3.__getstate__()
+    with pytest.raises((AttributeError, NotImplementedError, TypeError)):
+        pickle.dumps(s3)
 
 
 def test_cache_after_copy(s3):
@@ -1297,6 +1273,16 @@ def test_autocommit(s3):
         fo.commit()
 
 
+def test_autocommit_mpu(s3):
+    """When not autocommitting we always want to use multipart uploads"""
+    path = test_bucket_name + '/auto_commit_with_mpu'
+    with s3.open(path, 'wb', autocommit=True) as fo:
+        fo.write(b'1')
+    # fo.flush()
+    assert fo.mpu is not None
+    assert len(fo.parts) == 1
+
+
 def test_touch(s3):
     # create
     fn = test_bucket_name + "/touched"
@@ -1338,3 +1324,28 @@ def test_seek_reads(s3):
         size = 17562187
         d3 = f.read(size)
         assert len(d3) == size
+
+
+def test_connect_many():
+    from multiprocessing.pool import ThreadPool
+
+    def task(i):
+        S3FileSystem(anon=False).ls("")
+        return True
+
+    pool = ThreadPool(processes=20)
+    out = pool.map(task, range(40))
+    assert all(out)
+    pool.close()
+    pool.join()
+
+
+def test_requester_pays():
+    fn = test_bucket_name + "/myfile"
+    with moto.mock_s3():
+        s3 = S3FileSystem(requester_pays=True)
+        assert s3.req_kw["RequestPayer"] == "requester"
+        s3.mkdir(test_bucket_name)
+        s3.touch(fn)
+        with s3.open(fn, "rb") as f:
+            assert f.req_kw["RequestPayer"] == "requester"
